clear all
close all force
clc

len = 11e2;
dt = 1;
t = 0:dt:dt*(len-1); % [s]
Tinit = 273.15+29;   % [K] initial temperature of everything
Tenv = 273.15+22;    % [K] environment temperature 
% T_h_bulk = Tinit; % [K] bulk temperature of the hot plate
T_bot = nan(1,len);
T_bot(1) = Tinit;
T_top = nan(1,len);%ADDED BY SAMMY
T_top(1) = Tinit; %ADDED BY SAMMY

qin = nan(1,len);
qcool = nan(1,len);

Td = 66+273.15;
kp = 18;
ki = 0.4;
kd = 400; %100;

% T_hc
% T_c_bulk
% T_cf
% T_f_bulk

diam = 18e-2;
height = 5e-2; %3e-2;
volume = pi*height*(diam/2)^2;
density_water = 1000; % [kg/m^3]
m_obj = density_water*volume; % pi kilograms % [kg] mass of water 
c_obj = 4183; % [J/(kg*K)] specific heat capacity of Aluminium
SA_obj = pi*diam*height + pi*(diam/2)^2; % [m^2] surface area to environment
A_obj = pi*(diam/2)^2;

%h_air = 30; % [W/(m^2*K)] heat transfer coefficient
h_air = 10; % [W/(m^2*K)] heat transfer coefficient

% heat loss to environment is Newton's Law of Cooling

% Conduction: say Qin =~ Qcond through hotplate
% qcond = -k*(T_bot - T_top)/thickness=~ -k*(T_h_bot - T_h_top)/thickness
% k for aluminum ~= 220 W/mK
% say thickness d = 0.001 m
k = 0.6; % [W/(m * K)]
% T_top(i) = T_h_bot(i-1) + Qin(i)*d/(A_h*k)

qin(1:end) = 0;
error = zeros(1,len);
qp = nan(1,len);
qi = nan(1,len);
qd = nan(1,len);
accerror = 0;

for i=2:len

    error(i) = Td - T_bot(i-1);
    if sign(error(i))==sign(error(i-1))
        accerror = 1*accerror + error(i);
    else
        accerror = 0;
    end
    
        qp(i) = kp*error(i);
    qi(i) = ki*accerror;
    qd(i) = kd*diff(error(i-1:i));
  
    qin(i) = qp(i)+qi(i)+qd(i);
    qsave(i) = qin(i);
    
    if (qin(i) > 750)
        qin(i) = 750;
    elseif (qin(i) < 0)
        qin(i) = 0;
    end
    
    % Newton's law of cooling, loosing heat to environment
    % [W] = [W/(m^2*K)][m^2][K] ==> [W] check!
    qcool(i) = h_air*SA_obj*(T_bot(i-1) - Tenv);
%     qcool(i) = 0; %!!!
    
    % change in temperature based on heat/temperature relation
    % [K] = [s]*[W]*[kg^-1]*[(kg*K)/J] ==> [s*W*K/J] ==> [K]
    delT = dt*(qin(i)-qcool(i))/(m_obj*c_obj);
    T_bot(i) = T_bot(i-1) + delT;
    
    % conduction equation
    % [K] = [K] - [W]*[m]*[m^-2]*[m*K*W^-1]
    T_top(i) = T_bot(i) - height*qin(i)/(A_obj*k);
    T_top(i) = T_bot(i); %!!!
    
end

figure, plot(t,T_bot,'g'), hold on, plot(t,T_top,'k')
title('Th top and bot'), legend('bot','top')
figure, plot(t,qin,'r'), hold on, plot(t, qcool,'b'), hold off;
title('Heat I/O'), legend('Qin','Qcool');

figure, plot(t,qp,'r'), hold on
plot(t,qi,'g'), plot(t,qd,'b'), hold off
legend('qp','qi','qd')
ylim([-10, 710])

figure, plot(t,error), title('error')

% experiment 1:
% medium saucepan, water with 3cm depth, 18cm diameter
% hot plate on 'High' setting
% start to boil on surface at 4.5min
% rolling boil at 7min

%% 

uQ = uint16(qsave);
Qmax = uint16(700);
maxpos = uint16(30);
minpos = uint16(0);
pos = uQ*(maxpos - minpos)/Qmax + minpos;
pos(pos > maxpos) = maxpos;
figure, stem(pos);


